//
// jMax
// Copyright (C) 1994, 1995, 1998, 1999 by IRCAM-Centre Georges Pompidou, Paris, France.
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation; either version 2.1
// of the License, or (at your option) any later version.
// 
// See file COPYING.LIB for further informations on licensing terms.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Threading;

namespace ircam.jmax.dialogs
{
	/// <summary>
	/// Summary description for SplashDialog.
	/// </summary>
	public class SplashDialog : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Label versionLabel;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public SplashDialog()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
			this.versionLabel.Text = JMaxVersion.Version;
			InitializeThreadTimer();
		}

		private const int SPLASH_SHOW_DURATION = 4000;

		private TimerState timerState;
		private System.Threading.Timer timer;

		class TimerState 
		{
			public int counter = 0;
			public System.Threading.Timer tm;
		}

		private void InitializeThreadTimer()
		{
			timerState = new TimerState();

			// Create the delegate that invokes methods for the timer.
			TimerCallback timerDelegate = new TimerCallback(StartThread);
			
			// Create a timer that waits four second, then starts the timer immediately.
			timer = new System.Threading.Timer(timerDelegate, timerState, 0, SPLASH_SHOW_DURATION);
			
			// Keep a handle to the timer, so it can be disposed.
			timerState.tm = timer;
		}

		private void StartThread(object state)
		{
			timerState =(TimerState)state;
			timerState.counter++;

			if (timerState.counter == 1)
				this.ShowDialog();
			else
				DisposeSplash();
		}

		private void DisposeSplash()
		{
			this.Close();
			this.Dispose();
			timerState.tm.Dispose();
			timerState.tm = null;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(SplashDialog));
			this.versionLabel = new System.Windows.Forms.Label();
			this.SuspendLayout();
			// 
			// versionLabel
			// 
			this.versionLabel.BackColor = System.Drawing.Color.Transparent;
			this.versionLabel.Location = new System.Drawing.Point(16, 16);
			this.versionLabel.Name = "versionLabel";
			this.versionLabel.Size = new System.Drawing.Size(152, 16);
			this.versionLabel.TabIndex = 0;
			// 
			// SplashDialog
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.BackgroundImage = ((System.Drawing.Bitmap)(resources.GetObject("$this.BackgroundImage")));
			this.ClientSize = new System.Drawing.Size(400, 300);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.versionLabel});
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.None;
			this.Name = "SplashDialog";
			this.ShowInTaskbar = false;
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
			this.Text = "SplashDialog";
			this.KeyDown += new System.Windows.Forms.KeyEventHandler(this.SplashDialog_KeyDown);
			this.Click += new System.EventHandler(this.SplashDialog_Click);
			this.ResumeLayout(false);

		}
		#endregion

		private void SplashDialog_KeyDown(object sender, System.Windows.Forms.KeyEventArgs e)
		{
			DisposeSplash();
		}

		private void SplashDialog_Click(object sender, System.EventArgs e)
		{
			DisposeSplash();
		}
	}
}
