//
// jMax
// Copyright (C) 1994, 1995, 1998, 1999 by IRCAM-Centre Georges Pompidou, Paris, France.
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation; either version 2.1
// of the License, or (at your option) any later version.
// 
// See file COPYING.LIB for further informations on licensing terms.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

using System;
using System.IO;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

using ircam.fts.client;
using ircam.jmax.fts;
using ircam.jmax.dialogs;
using ircam.jmax.editors.patcher;
using ircam.jmax.editors.project;
using ircam.jmax.editors.configuration;

namespace ircam.jmax.editors.console
{
	/// <summary>
	/// Summary description for ConsoleWindow.
	/// </summary>
	public class ConsoleWindow : System.Windows.Forms.Form
	{
		private System.Windows.Forms.MainMenu mainMenu;
		private System.Windows.Forms.ToolBarButton newToolBarButton;
		private System.Windows.Forms.ImageList toolBarImageList;
		private System.Windows.Forms.ToolBarButton dspToolBarButton;
		private System.Windows.Forms.ToolBarButton openToolBarButton;
		private System.Windows.Forms.ToolBarButton separatorToolBarButton1;
		private System.Windows.Forms.ToolBarButton separatorToolBarButton2;
		private System.Windows.Forms.OpenFileDialog openFileDialog;
		private System.Windows.Forms.OpenFileDialog openProjectFileDialog;
		private System.Windows.Forms.OpenFileDialog openPackageFileDialog;
		private System.Windows.Forms.OpenFileDialog openConfigFileDialog;
		private System.Windows.Forms.ToolBarButton helpToolBarButton;
		private System.Windows.Forms.TextBox areaTextBox;
		private System.Windows.Forms.ToolBar controlPanelToolBar;
		private System.Windows.Forms.MenuItem fileMenuItem;
		private System.Windows.Forms.MenuItem newPatcherMenuItem;
		private System.Windows.Forms.MenuItem openMenuItem;
		private System.Windows.Forms.MenuItem fileSeparatorMenuItem1;
		private System.Windows.Forms.MenuItem DspMenuItem;
		private System.Windows.Forms.MenuItem fileSeparatorMenuItem2;
		private System.Windows.Forms.MenuItem recentFilesMenuItem;
		private System.Windows.Forms.MenuItem fileSeparatorMenuItem3;
		private System.Windows.Forms.MenuItem exitMenuItem;
		private System.Windows.Forms.MenuItem projectMenuItem;
		private System.Windows.Forms.MenuItem newProjectMenuItem;
		private System.Windows.Forms.MenuItem openProjectMenuItem;
		private System.Windows.Forms.MenuItem editProjectMenuItem;
		private System.Windows.Forms.MenuItem newPackageMenuItem;
		private System.Windows.Forms.MenuItem openPackageMenuItem;
		private System.Windows.Forms.MenuItem newConfigMenuItem;
		private System.Windows.Forms.MenuItem openConfigMenuItem;
		private System.Windows.Forms.MenuItem editConfigMenuItem;
		private System.Windows.Forms.MenuItem projectSeparatorMenuItem1;
		private System.Windows.Forms.MenuItem projectSeparatorMenuItem2;
		private System.Windows.Forms.MenuItem windowMenuItem;
		private System.Windows.Forms.MenuItem alwaysOnTopMenuItem;
		private System.Windows.Forms.MenuItem errorsMenuItem;
		private System.Windows.Forms.MenuItem helpMenuItem;
		private System.Windows.Forms.MenuItem contentsMenuItem;
		private System.Windows.Forms.MenuItem helpSeparatorMenuItem1;
		private System.Windows.Forms.MenuItem aboutMenuItem;
		private System.Windows.Forms.ToolBarButton separatorToolBarButton3;
		private System.Windows.Forms.ToolBarButton errorToolBarButton;
		private System.Windows.Forms.ToolBarButton topToolBarButton;
		private System.Windows.Forms.ToolBarButton printDsptoolBarButton;
		private System.ComponentModel.IContainer components;

		public ConsoleWindow()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
			if ((JMaxLibrary.GetEnvironmentVariables("debug") == null) || 
				((JMaxLibrary.GetEnvironmentVariables("debug")).Equals("true"))) 
			{
				this.Text += " - Debug Mode";
				this.printDsptoolBarButton.Visible = true;
			}
		}

		private string helpFile = "jmax.chm";
		private FtsDspControl control;
		
		// (fd) This lock prevents a "setSelected" ont the dspOnButton to cause
		// "itemStateChanged" to repropagates the value to FTS
		private bool lockMode = false;

		internal void NewPatcher()
		{
			FtsPatcherObject patcher = null;

			try 
			{
				patcher = new FtsPatcherObject();
			}
			catch (IOException e) 
			{
				Console.Error.WriteLine("[NewAction]: Error in FtsPatcherObject creation!");
				Console.Error.WriteLine(e.StackTrace);
			}

			if (patcher != null) 
			{
				patcher.RequestOpenEditor();
				patcher.RequestStopWaiting();
			}
		}

		private void Open()
		{
			string fileName;
			
			fileName = MaxFileChooser.ChooseFileToOpen(this);
			
			if (fileName != null) 
			{
				RecentFileHistory recentFileHistory = JMaxLibrary.RecentFileHistory;
				recentFileHistory.AddFile(new FileInfo(fileName));
				
				try 
				{
					JMaxLibrary.Load(fileName);
				}
				catch (IOException e) 
				{
					Console.Error.WriteLine("[OpenAction]: I/O error loading file " + fileName);
					Console.Error.WriteLine(e.StackTrace);
				}
			}
		}

		private void About()
		{		
			AboutDialog ad = new AboutDialog();
			ad.ShowDialog(this);
			ad.Dispose();						
		}

		public string AreaAppendText
		{
			set
			{
				areaTextBox.AppendText(value + "\r\n");
			}
		}

		public bool DspOn
		{
			get
			{
				return dspToolBarButton.Pushed;
			}
			set
			{
				dspToolBarButton.Pushed = value;
			}
		}

		public void Init()
		{
			try 
			{
				control = new FtsDspControl();
				control.Send(FtsSymbol.GetSymbol("upload"));
			}
			catch (IOException e) 
			{
				Console.Error.WriteLine("ControlPanel: Error in FtsDspControl creation!");
				Console.Error.WriteLine(e.StackTrace);
			}
			
			if (JMaxLibrary.GetEnvironmentVariables("debug") != null) {}

			if (!lockMode) 
			{
				if (!DspOn) 
				{
					control.RequestSetDspOn(false);
				}
				else 
				{
					control.RequestSetDspOn(true);
				}
			}
		}

		public bool LockMode
		{
			get
			{
				return lockMode;
			}
			set
			{
				lockMode = value;
			}
		}

		public FtsDspControl DspControl
		{
			get
			{
				return control;
			}
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(ConsoleWindow));
			this.mainMenu = new System.Windows.Forms.MainMenu();
			this.fileMenuItem = new System.Windows.Forms.MenuItem();
			this.newPatcherMenuItem = new System.Windows.Forms.MenuItem();
			this.openMenuItem = new System.Windows.Forms.MenuItem();
			this.fileSeparatorMenuItem1 = new System.Windows.Forms.MenuItem();
			this.DspMenuItem = new System.Windows.Forms.MenuItem();
			this.fileSeparatorMenuItem2 = new System.Windows.Forms.MenuItem();
			this.recentFilesMenuItem = new System.Windows.Forms.MenuItem();
			this.fileSeparatorMenuItem3 = new System.Windows.Forms.MenuItem();
			this.exitMenuItem = new System.Windows.Forms.MenuItem();
			this.projectMenuItem = new System.Windows.Forms.MenuItem();
			this.newProjectMenuItem = new System.Windows.Forms.MenuItem();
			this.openProjectMenuItem = new System.Windows.Forms.MenuItem();
			this.editProjectMenuItem = new System.Windows.Forms.MenuItem();
			this.projectSeparatorMenuItem1 = new System.Windows.Forms.MenuItem();
			this.newPackageMenuItem = new System.Windows.Forms.MenuItem();
			this.openPackageMenuItem = new System.Windows.Forms.MenuItem();
			this.projectSeparatorMenuItem2 = new System.Windows.Forms.MenuItem();
			this.newConfigMenuItem = new System.Windows.Forms.MenuItem();
			this.openConfigMenuItem = new System.Windows.Forms.MenuItem();
			this.editConfigMenuItem = new System.Windows.Forms.MenuItem();
			this.windowMenuItem = new System.Windows.Forms.MenuItem();
			this.alwaysOnTopMenuItem = new System.Windows.Forms.MenuItem();
			this.errorsMenuItem = new System.Windows.Forms.MenuItem();
			this.helpMenuItem = new System.Windows.Forms.MenuItem();
			this.contentsMenuItem = new System.Windows.Forms.MenuItem();
			this.helpSeparatorMenuItem1 = new System.Windows.Forms.MenuItem();
			this.aboutMenuItem = new System.Windows.Forms.MenuItem();
			this.controlPanelToolBar = new System.Windows.Forms.ToolBar();
			this.newToolBarButton = new System.Windows.Forms.ToolBarButton();
			this.openToolBarButton = new System.Windows.Forms.ToolBarButton();
			this.separatorToolBarButton1 = new System.Windows.Forms.ToolBarButton();
			this.errorToolBarButton = new System.Windows.Forms.ToolBarButton();
			this.topToolBarButton = new System.Windows.Forms.ToolBarButton();
			this.separatorToolBarButton2 = new System.Windows.Forms.ToolBarButton();
			this.helpToolBarButton = new System.Windows.Forms.ToolBarButton();
			this.separatorToolBarButton3 = new System.Windows.Forms.ToolBarButton();
			this.dspToolBarButton = new System.Windows.Forms.ToolBarButton();
			this.printDsptoolBarButton = new System.Windows.Forms.ToolBarButton();
			this.toolBarImageList = new System.Windows.Forms.ImageList(this.components);
			this.areaTextBox = new System.Windows.Forms.TextBox();
			this.openProjectFileDialog = new System.Windows.Forms.OpenFileDialog();
			this.openPackageFileDialog = new System.Windows.Forms.OpenFileDialog();
			this.openConfigFileDialog = new System.Windows.Forms.OpenFileDialog();
			this.openFileDialog = new System.Windows.Forms.OpenFileDialog();
			this.SuspendLayout();
			// 
			// mainMenu
			// 
			this.mainMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					 this.fileMenuItem,
																					 this.projectMenuItem,
																					 this.windowMenuItem,
																					 this.helpMenuItem});
			// 
			// fileMenuItem
			// 
			this.fileMenuItem.Index = 0;
			this.fileMenuItem.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																						 this.newPatcherMenuItem,
																						 this.openMenuItem,
																						 this.fileSeparatorMenuItem1,
																						 this.DspMenuItem,
																						 this.fileSeparatorMenuItem2,
																						 this.recentFilesMenuItem,
																						 this.fileSeparatorMenuItem3,
																						 this.exitMenuItem});
			this.fileMenuItem.Text = "&File";
			// 
			// newPatcherMenuItem
			// 
			this.newPatcherMenuItem.Index = 0;
			this.newPatcherMenuItem.Shortcut = System.Windows.Forms.Shortcut.CtrlN;
			this.newPatcherMenuItem.Text = "&New Patcher";
			this.newPatcherMenuItem.Click += new System.EventHandler(this.newPatcherMenuItem_Click);
			// 
			// openMenuItem
			// 
			this.openMenuItem.Index = 1;
			this.openMenuItem.Shortcut = System.Windows.Forms.Shortcut.CtrlO;
			this.openMenuItem.Text = "&Open";
			this.openMenuItem.Click += new System.EventHandler(this.openMenuItem_Click);
			// 
			// fileSeparatorMenuItem1
			// 
			this.fileSeparatorMenuItem1.Index = 2;
			this.fileSeparatorMenuItem1.Text = "-";
			// 
			// DspMenuItem
			// 
			this.DspMenuItem.Checked = true;
			this.DspMenuItem.Enabled = false;
			this.DspMenuItem.Index = 3;
			this.DspMenuItem.Shortcut = System.Windows.Forms.Shortcut.CtrlDel;
			this.DspMenuItem.Text = "&DSP";
			// 
			// fileSeparatorMenuItem2
			// 
			this.fileSeparatorMenuItem2.Index = 4;
			this.fileSeparatorMenuItem2.Text = "-";
			// 
			// recentFilesMenuItem
			// 
			this.recentFilesMenuItem.Enabled = false;
			this.recentFilesMenuItem.Index = 5;
			this.recentFilesMenuItem.Text = "Recent &Files";
			// 
			// fileSeparatorMenuItem3
			// 
			this.fileSeparatorMenuItem3.Index = 6;
			this.fileSeparatorMenuItem3.Text = "-";
			// 
			// exitMenuItem
			// 
			this.exitMenuItem.Index = 7;
			this.exitMenuItem.Shortcut = System.Windows.Forms.Shortcut.CtrlQ;
			this.exitMenuItem.Text = "E&xit";
			this.exitMenuItem.Click += new System.EventHandler(this.exitMenuItem_Click);
			// 
			// projectMenuItem
			// 
			this.projectMenuItem.Index = 1;
			this.projectMenuItem.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																							this.newProjectMenuItem,
																							this.openProjectMenuItem,
																							this.editProjectMenuItem,
																							this.projectSeparatorMenuItem1,
																							this.newPackageMenuItem,
																							this.openPackageMenuItem,
																							this.projectSeparatorMenuItem2,
																							this.newConfigMenuItem,
																							this.openConfigMenuItem,
																							this.editConfigMenuItem});
			this.projectMenuItem.Text = "&Project";
			// 
			// newProjectMenuItem
			// 
			this.newProjectMenuItem.Index = 0;
			this.newProjectMenuItem.Text = "&New Project";
			this.newProjectMenuItem.Click += new System.EventHandler(this.newProjectMenuItem_Click);
			// 
			// openProjectMenuItem
			// 
			this.openProjectMenuItem.Index = 1;
			this.openProjectMenuItem.Text = "&Open Project...";
			this.openProjectMenuItem.Click += new System.EventHandler(this.openProjectMenuItem_Click);
			// 
			// editProjectMenuItem
			// 
			this.editProjectMenuItem.Index = 2;
			this.editProjectMenuItem.Text = "&Edit Project...";
			this.editProjectMenuItem.Click += new System.EventHandler(this.editProjectMenuItem_Click);
			// 
			// projectSeparatorMenuItem1
			// 
			this.projectSeparatorMenuItem1.Index = 3;
			this.projectSeparatorMenuItem1.Text = "-";
			// 
			// newPackageMenuItem
			// 
			this.newPackageMenuItem.Index = 4;
			this.newPackageMenuItem.Text = "New &Package";
			// 
			// openPackageMenuItem
			// 
			this.openPackageMenuItem.Index = 5;
			this.openPackageMenuItem.Text = "Open Pa&ckage...";
			// 
			// projectSeparatorMenuItem2
			// 
			this.projectSeparatorMenuItem2.Index = 6;
			this.projectSeparatorMenuItem2.Text = "-";
			// 
			// newConfigMenuItem
			// 
			this.newConfigMenuItem.Index = 7;
			this.newConfigMenuItem.Text = "New Con&figuration";
			// 
			// openConfigMenuItem
			// 
			this.openConfigMenuItem.Index = 8;
			this.openConfigMenuItem.Text = "Open Confi&guration...";
			// 
			// editConfigMenuItem
			// 
			this.editConfigMenuItem.Index = 9;
			this.editConfigMenuItem.Text = "Edit &Configuration...";
			// 
			// windowMenuItem
			// 
			this.windowMenuItem.Index = 2;
			this.windowMenuItem.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																						   this.alwaysOnTopMenuItem,
																						   this.errorsMenuItem});
			this.windowMenuItem.Text = "&Window";
			// 
			// alwaysOnTopMenuItem
			// 
			this.alwaysOnTopMenuItem.Checked = true;
			this.alwaysOnTopMenuItem.Index = 0;
			this.alwaysOnTopMenuItem.Text = "&Always on Top";
			this.alwaysOnTopMenuItem.Click += new System.EventHandler(this.alwaysOnTopMenuItem_Click);
			// 
			// errorsMenuItem
			// 
			this.errorsMenuItem.Enabled = false;
			this.errorsMenuItem.Index = 1;
			this.errorsMenuItem.Text = "&Errors";
			// 
			// helpMenuItem
			// 
			this.helpMenuItem.Index = 3;
			this.helpMenuItem.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																						 this.contentsMenuItem,
																						 this.helpSeparatorMenuItem1,
																						 this.aboutMenuItem});
			this.helpMenuItem.Text = "&Help";
			// 
			// contentsMenuItem
			// 
			this.contentsMenuItem.Index = 0;
			this.contentsMenuItem.Shortcut = System.Windows.Forms.Shortcut.F1;
			this.contentsMenuItem.Text = "&Contents...";
			this.contentsMenuItem.Click += new System.EventHandler(this.contentsMenuItem_Click);
			// 
			// helpSeparatorMenuItem1
			// 
			this.helpSeparatorMenuItem1.Index = 1;
			this.helpSeparatorMenuItem1.Text = "-";
			// 
			// aboutMenuItem
			// 
			this.aboutMenuItem.Index = 2;
			this.aboutMenuItem.Text = "&About jMax...";
			this.aboutMenuItem.Click += new System.EventHandler(this.aboutMenuItem_Click);
			// 
			// controlPanelToolBar
			// 
			this.controlPanelToolBar.Appearance = System.Windows.Forms.ToolBarAppearance.Flat;
			this.controlPanelToolBar.Buttons.AddRange(new System.Windows.Forms.ToolBarButton[] {
																								   this.newToolBarButton,
																								   this.openToolBarButton,
																								   this.separatorToolBarButton1,
																								   this.errorToolBarButton,
																								   this.topToolBarButton,
																								   this.separatorToolBarButton2,
																								   this.helpToolBarButton,
																								   this.separatorToolBarButton3,
																								   this.dspToolBarButton,
																								   this.printDsptoolBarButton});
			this.controlPanelToolBar.ButtonSize = new System.Drawing.Size(16, 15);
			this.controlPanelToolBar.DropDownArrows = true;
			this.controlPanelToolBar.ImageList = this.toolBarImageList;
			this.controlPanelToolBar.Location = new System.Drawing.Point(0, 0);
			this.controlPanelToolBar.Name = "controlPanelToolBar";
			this.controlPanelToolBar.ShowToolTips = true;
			this.controlPanelToolBar.Size = new System.Drawing.Size(352, 28);
			this.controlPanelToolBar.TabIndex = 3;
			this.controlPanelToolBar.ButtonClick += new System.Windows.Forms.ToolBarButtonClickEventHandler(this.controlPanelToolBar_ButtonClick);
			// 
			// newToolBarButton
			// 
			this.newToolBarButton.ImageIndex = 0;
			this.newToolBarButton.ToolTipText = "New Patcher";
			// 
			// openToolBarButton
			// 
			this.openToolBarButton.ImageIndex = 1;
			this.openToolBarButton.ToolTipText = "Open";
			// 
			// separatorToolBarButton1
			// 
			this.separatorToolBarButton1.Style = System.Windows.Forms.ToolBarButtonStyle.Separator;
			// 
			// errorToolBarButton
			// 
			this.errorToolBarButton.ImageIndex = 3;
			this.errorToolBarButton.ToolTipText = "Error";
			// 
			// topToolBarButton
			// 
			this.topToolBarButton.ImageIndex = 4;
			this.topToolBarButton.Style = System.Windows.Forms.ToolBarButtonStyle.ToggleButton;
			this.topToolBarButton.ToolTipText = "Always on top";
			// 
			// separatorToolBarButton2
			// 
			this.separatorToolBarButton2.Style = System.Windows.Forms.ToolBarButtonStyle.Separator;
			// 
			// helpToolBarButton
			// 
			this.helpToolBarButton.ImageIndex = 2;
			this.helpToolBarButton.ToolTipText = "Help";
			// 
			// separatorToolBarButton3
			// 
			this.separatorToolBarButton3.Style = System.Windows.Forms.ToolBarButtonStyle.Separator;
			// 
			// dspToolBarButton
			// 
			this.dspToolBarButton.ImageIndex = 5;
			this.dspToolBarButton.Style = System.Windows.Forms.ToolBarButtonStyle.ToggleButton;
			this.dspToolBarButton.ToolTipText = "DSP";
			// 
			// printDsptoolBarButton
			// 
			this.printDsptoolBarButton.ImageIndex = 6;
			this.printDsptoolBarButton.ToolTipText = "Print Dsp Chain";
			this.printDsptoolBarButton.Visible = false;
			// 
			// toolBarImageList
			// 
			this.toolBarImageList.ColorDepth = System.Windows.Forms.ColorDepth.Depth32Bit;
			this.toolBarImageList.ImageSize = new System.Drawing.Size(16, 16);
			this.toolBarImageList.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("toolBarImageList.ImageStream")));
			this.toolBarImageList.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// areaTextBox
			// 
			this.areaTextBox.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.areaTextBox.AutoSize = false;
			this.areaTextBox.BackColor = System.Drawing.SystemColors.Window;
			this.areaTextBox.HideSelection = false;
			this.areaTextBox.Location = new System.Drawing.Point(0, 24);
			this.areaTextBox.Multiline = true;
			this.areaTextBox.Name = "areaTextBox";
			this.areaTextBox.ReadOnly = true;
			this.areaTextBox.ScrollBars = System.Windows.Forms.ScrollBars.Vertical;
			this.areaTextBox.Size = new System.Drawing.Size(352, 390);
			this.areaTextBox.TabIndex = 4;
			this.areaTextBox.Text = "";
			this.areaTextBox.WordWrap = false;
			// 
			// ConsoleWindow
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(352, 413);
			this.Controls.Add(this.areaTextBox);
			this.Controls.Add(this.controlPanelToolBar);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Menu = this.mainMenu;
			this.Name = "ConsoleWindow";
			this.Text = "jMax Console";
			this.ResumeLayout(false);

		}
		#endregion

		private void controlPanelToolBar_ButtonClick(object sender, System.Windows.Forms.ToolBarButtonClickEventArgs e)
		{
			if(e.Button == this.newToolBarButton)	
			{
				NewPatcher();
			}
			else if(e.Button == this.openToolBarButton)
			{
				Open();
			}
			else if(e.Button == this.printDsptoolBarButton)
			{
				control.DspPrint();
			}
		}
	
		private void newPatcherMenuItem_Click(object sender, System.EventArgs e)
		{
			NewPatcher();
		}

		private void openMenuItem_Click(object sender, System.EventArgs e)
		{
			Open();
		}

		private void exitMenuItem_Click(object sender, System.EventArgs e)
		{
			JMaxLibrary.Quit();
		}

		private void newProjectMenuItem_Click(object sender, System.EventArgs e)
		{
			ProjectEditor.NewProject();
		}

		private void openProjectMenuItem_Click(object sender, System.EventArgs e)
		{
			ProjectEditor.OpenProject();
		}

		private void editProjectMenuItem_Click(object sender, System.EventArgs e)
		{
			JMaxLibrary.Project.RequestOpenEditor();
		}

		private void alwaysOnTopMenuItem_Click(object sender, System.EventArgs e)
		{
			this.TopMost = !this.TopMost;
		}

		private void contentsMenuItem_Click(object sender, System.EventArgs e)
		{
			Help.ShowHelp(this, helpFile);
		}

		private void aboutMenuItem_Click(object sender, System.EventArgs e)
		{
			About();		
		}
	}
}
