//
// jMax
// Copyright (C) 1994, 1995, 1998, 1999 by IRCAM-Centre Georges Pompidou, Paris, France.
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// See file LICENSE for further informations on licensing terms.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 

using System;
using System.Drawing;

using ircam.jmax.editors.patcher;
using ircam.jmax.editors.patcher.objects;

namespace ircam.jmax.editors.patcher.interactions
{
	/// <summary>The interaction handling drag-select; 
	/// started by a click on the background;
	/// Note that since the proper event sequence is guaranteed by the
	/// input system (i.e. Down [Drag]* Up), we don't need to keep track
	/// of the status here, so we have no real state-machine.
	/// </summary>
	
	internal class HResizeInteraction : Interaction
	{
		internal GraphicObject obj;

		internal override void configureInputFilter(InteractionEngine filter)
		{
			filter.isFollowingMoves = true;
			filter.isAutoScrolling = true;
		}

		private class ResizeTrueObjectAction : IObjectAction
		{
			public void processObject(GraphicObject obj)
			{
				obj.resizing(true);
			}
		}

		
		private class WidthObjectAction : IObjectAction
		{
			public WidthObjectAction(HResizeInteraction hresizeInteraction)
			{
				this.hresizeInteraction = hresizeInteraction;
			}

			public void processObject(GraphicObject obj)
			{
				obj.Redraw();
				obj.redrawConnections();
				obj.Width = (obj.Width + hresizeInteraction.dx);
				obj.Redraw();
				obj.redrawConnections();
			}

			private HResizeInteraction hresizeInteraction;
		}

		private class WidthShiftObjectAction : IObjectAction
		{
			public WidthShiftObjectAction(HResizeInteraction hresizeInteraction)
			{
				this.hresizeInteraction = hresizeInteraction;
			}

			public void processObject(GraphicObject obj)
			{
				obj.Redraw();
				obj.redrawConnections();
				obj.WidthShift = obj.Width+ hresizeInteraction.dx;
				obj.Redraw();
				obj.redrawConnections();
			}

			private HResizeInteraction hresizeInteraction;
		}

		private class ResizeFalseObjectAction : IObjectAction
		{
			public void processObject(GraphicObject obj)
			{
				obj.resizing(false);
			}
		}

		internal int dx;
		internal override void gotSqueack(ErmesSketchPad editor, int squeack, SensibilityArea area, Point mouse, Point oldMouse)
		{
			if (Squeack.isDown(squeack) && Squeack.onHResizeHandle(squeack))
			{
				obj = (GraphicObject) area.Target;
//				editor.Cursor = System.Windows.Forms.Cursors.SizeWE;
				
				if ((!obj.isSelected) || ((obj.isSelected) && (ErmesSelection.patcherSelection.isSingleton())))
				{
					obj.resizing(true);
					editor.setUndo("Resize", obj, false, false);
				}
				else
				{
					if (ErmesSelection.patcherSelection.ownedBy(editor))
						ErmesSelection.patcherSelection.apply(new ResizeTrueObjectAction());
					editor.setUndo("Resize", false, false);
				}
			}
			else if (Squeack.isDrag(squeack))
			{
				if (!Squeack.isShift(squeack))
				{
					if ((!obj.isSelected) || ((obj.isSelected) && (ErmesSelection.patcherSelection.isSingleton())))
					{
						obj.Redraw();
						obj.redrawConnections();
						obj.Width = (mouse.X - obj.X);
						obj.Redraw();
						obj.redrawConnections();
					}
					else
					{
						if (ErmesSelection.patcherSelection.ownedBy(editor))
						{
							dx = mouse.X - oldMouse.X;
							ErmesSelection.patcherSelection.apply(new WidthObjectAction(this));
						}
					}
				}
				else
				{
					if ((!obj.isSelected) || ((obj.isSelected) && (ErmesSelection.patcherSelection.isSingleton())))
					{
						obj.Redraw();
						obj.redrawConnections();
						obj.WidthShift = mouse.X - obj.X;
						obj.Redraw();
						obj.redrawConnections();
					}
					else
					{
						if (ErmesSelection.patcherSelection.ownedBy(editor))
						{
							dx = mouse.X - oldMouse.X;
							ErmesSelection.patcherSelection.apply(new WidthShiftObjectAction(this));
						}
					}
				}
				editor.fixSize();
			}
			else if (Squeack.isUp(squeack))
			{
				if ((!obj.isSelected) || ((obj.isSelected) && (ErmesSelection.patcherSelection.isSingleton())))
					obj.resizing(false);
				else if (ErmesSelection.patcherSelection.ownedBy(editor))
					ErmesSelection.patcherSelection.apply(new ResizeFalseObjectAction());
				
//				editor.Cursor = System.Windows.Forms.Cursors.Default;
				editor.endInteraction();
			}
		}
	}
}